/*jslint browser: true, eqeqeq: false, evil: true, css: true, forin: true, fragment: true, nomen: false, onevar: false, plusplus: false, undef: true, white: false */
/*window, PFLEX, escape, unescape*/

// *** Header ***

(function () {

// *** START ***
// *** Make sure PFLEX lib doesn't get loaded multiple times on js ad *** 
if (typeof PFLEX == "undefined" || typeof PFLEX.render != "function") {
// *** START ***

// *** Common base.js***

//console.log("Initializing PFLEX");

var d = document,
    q = window;

PFLEX = {
    create: function() {
        function klass() {
            this.init.apply(this, arguments);      
        }
        if (!klass.prototype.init) {
            klass.prototype.init = function() { };
        }
        klass.prototype.constructor = klass;
        return klass;
    },
    $: function (element) {
        if (PFLEX.isString(element)) {
            element = document.getElementById(element);
        }
        return element;
    },
    $A: function (iterable) {
        if (!iterable) { 
            return []; 
        }
        if (PFLEX.browser.isWebkit) {
            if (!(PFLEX.isFunction(iterable) && iterable == '[object NodeList]') && iterable.toArray) {
                return iterable.toArray();
            }
        } else {
            if (iterable.toArray) {
                return iterable.toArray();
            }
        }
        var length = iterable.length || 0, results = new Array(length);
        while (length--) { 
            results[length] = iterable[length];
        }
        return results;
    },
    strip: function(value) {
        if (PFLEX.isString(value)) {
            return value.replace(/^\s+/, '').replace(/\s+$/, '');
        }
        return value;
    },
    parseInputName: function (n) {
        return (/.+:.+/.test(n)) ? n.split(":")[1] : n;
    },
    getFormElement: function (id) {
        var form = PFLEX.$(id);
        if (form) {
            if (form.tagName.toLowerCase() == "form") { 
                return form; 
            }
            if (form.getElementsByTagName("form").length > 0) {
                return form.getElementsByTagName("form")[0];
            }
        }
        return form;
    },
    isOfferSelected: function (input) {
        return (PFLEX.isOfferElement(input) && input.checked);
    },
    isOfferElement: function (input) {
        return (typeof input != "undefined" && input.type == "checkbox" && /.+:pflexoptin/.test(input.id));
    },
    hasClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        var elementClassName = element.className;
            return (elementClassName.length > 0 && (elementClassName == className || new RegExp("(^|\\s)" + className + "(\\s|$)").test(elementClassName)));
    },
    addClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        if (!PFLEX.hasClassName(element, className))
            element.className += (element.className ? ' ' : '') + className;
        return element;
    },
    removeClassName: function(element, className) {
        if (!(element = PFLEX.$(element))) return;
        element.className = PFLEX.strip(element.className.replace(new RegExp("(^|\\s+)" + className + "(\\s+|$)"), ' '));
        return element;
    },
    visible: function(element) {
        return PFLEX.$(element).style.display != 'none';
    },
    renderPixel: function (url, doBust, semaphore) {
        var fn;
        if (!url || url === "") { 
            return; 
        }
        var signal = function() {
            semaphore.signal();
        };
        if (typeof semaphore != "undefined" && typeof semaphore == "object") {
            fn = function() { signal(); return; };
        } else {
            fn = function() { return; };
        }
        if (url.indexOf("http") !== 0) {
            url = "http" + PFLEX.prefix + "://" + url;
        }
        if (doBust) {
            var c = (url.indexOf("?") > -1) ? "&" : "?";
            url += (c + Math.round(Math.random() * 2147483647));
        }
        var p = new Image(1, 1);
        p.onload = fn;
        p.onerror = p.onload;
        p.src = url;
        //PFLEX.log("Writing pixel:", url);
    },
        simpleXhr: function(url,successFn,errorFn){
    	var xhrObj = new XMLHttpRequest(),
    	readystatechangeFn = function(event){
    		if (xhrObj.readyState !== 4){ return true; }
    		if (xhrObj.status === 200){
    			successFn(xhrObj);
    		}else{
    			errorFn(xhrObj);
    		}
    	};
    	
	    xhrObj.open('GET', url, true);
        xhrObj.onreadystatechange = readystatechangeFn;
        xhrObj.send(); 
    	
    },
    submit: function (submitUrl, dealId, formData, customData, semaphore, subSourceId) {
        try {
            if (!/^\d+$/.test(dealId)) {
                return;
            }
        } catch(e) {}
        var data = PFLEX.extend(PFLEX.extend({}, formData), PFLEX.isUndefined(customData) ? {} : customData);
        if (PFLEX.isString(subSourceId)) data["pflexsubsrcid"] = subSourceId; 
        var url = submitUrl + "?pontiflexdealid=" + dealId;
        for (var v in data) {
            url += "&" + encodeURIComponent(v) + "=" + encodeURIComponent(data[v]);
        }
        PFLEX.renderPixel(url, false, semaphore);
    },
    addStylesheet: function (css) {
        if (!css || css === "") { return; }
        var s,h,i,b;
        if (PFLEX.browser.isIe) {
            s = d.createStyleSheet();
            s.cssText = css;
        } else {
            h = d.getElementsByTagName("head")[0];
            if (!h) {
                b = d.getElementsByTagName("body")[0];
                h = d.createElement("head");
                b.parentNode.insertBefore(h, b);
            }
            s = d.createElement("style");
            i = (PFLEX.browser.isWebkit) ? 'innerText' : 'innerHTML';
            s[i] = css;
            h.appendChild(s);
        }
    },
    createElement: function (tag, id, properties, attributes, styles) {
        var elm = null, value;
        if (PFLEX.isString(tag)) {
            elm = document.createElement(tag);
            if(id){
            	elm.id = id; 
            }
            for (value in properties) {
                elm[value] = properties[value]; 
            }            
            for (value in attributes) {
                elm.setAttribute(value, attributes[value]);
            }
            for (value in styles) {
                elm.style[value] = styles[value];
            }
        }
        return elm;
    },
    createEl: function(args){
    	// mk - required by following conditionals
    	if(!args.prop){ args.prop = {}; }
    	if(args.html){  args.prop.innerHTML = args.html; }
    	if(args["class"]){ args.prop["className"] = args["class"]; }
    	
    	return PFLEX.createElement.call(this, args.tag, args.id, args.prop, args.attr, args.style);
    },
    render: function (pid, oid, options) {
        if (!pid) { 
            return; 
        }

        if (!oid) {
            oid = null;
        }
        if (!options) {
            options = {}
        }

        var o = {
            options: options,
            oid: oid
        };

        for (var k in options) {
            var v;
            switch (k) {
            case "trackingFunction":
                var fn;
                try {
                    if (typeof options.trackingFunction === "function") {
                        fn = options.trackingFunction;
                    } else if (typeof options.trackingFunction === "string") {
                        fn = new Function(options.trackingFunction);
                    }
                } catch(e) { }
                if (typeof fn === "function") {
                    PFLEX.ActionHandler.add("doCustomUserTrack", fn);                   
                }     
                break;
            case "trackingURL":
                k = "trackingURL";
                v = PFLEX.cleanUrl(options.trackingURL);
                options.trackingURL = v;
                break;
            case "redirectURL":
                k = "redirectURL";
                v = PFLEX.cleanUrl(options.redirectURL);
                options.redirectURL = v;
                break;
            default:
                v = (options[k]) ? options[k] : null;
            }
            if (!v) { continue; }
            o[k] = v;
        }
        if (!o.data) {
            o.data = (typeof pontiflex_data != "undefined") ? PFLEX.extend({}, pontiflex_data) : {};
        }

        PFLEX.Ads.addPlacement(pid, o);
        PFLEX.Ads.addEvent(pid, o);
    },
    parseOptions: function() {
        return PFLEX.deserialize(location.search.substring(1));
    },
    bind: function(method) {
        if (arguments.length < 3 && PFLEX.isUndefined(arguments[1])) return method;
        var args = Array.prototype.slice.call(arguments), fn = args.shift(), object = args.shift();
        return function() {
            return fn.apply(object, args.concat(Array.prototype.slice.call(arguments)));
        };
    },
    isFunction: function(object) {
        return typeof object == "function";
    },
    isString: function(object) {
        return typeof object == "string";
    },
    isNumber: function(object) {
        return typeof object == "number";
    },
    isUndefined: function(object) {
        return typeof object == "undefined";
    },
    isElement: function(object) {
        return !!(object && object.nodeType == 1);
    },
    isInput: function(object) {
        return (PFLEX.isElement(object) && object.nodeName.toLowerCase() == 'input');
    },
    isArray: function(object) {
        return object != null && typeof object == "object" && 'splice' in object && 'join' in object;
    },
    isEmpty: function(object) {
        var prop;
        if (typeof object === 'object' || typeof object === 'function') {
            for (prop in object) {
                if (object.hasOwnProperty(prop)) { return false; }
            }
        }
        return true;
    },
    isOption: function(object) {
        if (PFLEX.isElement(object)) {
            if (!PFLEX.isUndefined(Object.type)) {
                var type  = object.type.toLowerCase();
                return (object.nodeName.toLowerCase() == 'input' || (type == 'radio' || type == 'checkbox'));
            }
        }
        return false;
    },
    isStringBlank: function(val) {
        return (!val || /^\s*$/.test(val));
    },
    isFormElement: function(object) {
        return (PFLEX.isElement(object) && (object.nodeName.toLowerCase() == 'input' ||
            object.nodeName.toLowerCase() == 'textarea' || object.nodeName.toLowerCase() == 'select'));
    },
    isDateField : function(object) {
        return (PFLEX.isElement(object) && object.getAttribute("pflexFieldType") == "date");
    },
    isDatePartField: function(object) {
        var fieldType;
        if (PFLEX.isElement(object)) {
            fieldType = object.getAttribute("pflexFieldType");
        }
        return (fieldType == "year" || fieldType == "month" || fieldType == "day");
    },
    extend: function(destination, source) {
        for (var property in source)
            destination[property] = source[property];
        return destination;
    },
    cleanUrl: function(url) {
        var v, uri;
        if (url && /^http/.test(url)) {
            if (/^http%3A/.test(url)) {
                uri = decodeURI(url);
            } else {
                uri = url;
            }
            v = uri.replace(/(https{0,1}:)(?!\/\/)/i, "$1\/\/");
        } else {
            v = null;
        }
        return v;
    },
    parseUri: function(str) {
        // parseUri 1.2.2
        // (c) Steven Levithan <stevenlevithan.com>
        // MIT License
        var o = {
            strictMode: false,
            key: ["source","protocol","authority","userInfo","user","password","host","port","relative","path","directory","file","query","anchor"],
            q: {  
                name:   "queryKey",
                parser: /(?:^|&)([^&=]*)=?([^&]*)/g
            },
            parser: {
                strict: /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/,
                loose:  /^(?:(?![^:@]+:[^:@\/]*@)([^:\/?#.]+):)?(?:\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?([^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/
            }
        },
        m = o.parser[o.strictMode ? "strict" : "loose"].exec(str),
        uri = {},
        i = 14;

        while (i--) uri[o.key[i]] = m[i] || "";

        uri[o.q.name] = {};
        uri[o.key[12]].replace(o.q.parser, function ($0, $1, $2) {
            if ($1) uri[o.q.name][$1] = $2;
        });
        return uri;
    },
    sleep: function (ms) {
        var start = new Date().getTime(), sleeping = true;
        while (sleeping) {
            if ((new Date().getTime() - start) > ms) {
                sleeping = false;
            }
        }
    },
    format: function(text) {
        if (arguments.length <= 1) {
            return text;
        }
        var count = arguments.length - 2;
        for (var token = 0; token <= count; token++) {
            text = text.replace(new RegExp("\\{" + token + "\\}", "gi"), arguments[token + 1]);
        }
        return text;
    },
    formatUrl: function(url, placement) {
        if (placement) {
            var name;
            //replace template variables like $first_name$, $last_name$ ...
            for(name in placement.formData) {
                url = url.replace(new RegExp("\\$" + name + "\\$", "gi"), encodeURIComponent(placement.formData[name]));
            }
            
            //replace template variables from options
            for(name in placement.options) {
                if (PFLEX.isNumber(placement.options[name]) || PFLEX.isString(placement.options[name])) {
                    url = url.replace(new RegExp("\\$pflex_o_" + name + "\\$", "gi"), encodeURIComponent(placement.options[name]));
                }
            };
            
            //replace random
            url = url.replace(new RegExp("\\$pflex_random\\$", "gi"),  Math.floor(Math.random()*1E16));
        }
        return url;
    },
    getFormDate: function(form, dateFieldName) {
        return form[dateFieldName + "_yyyy"].value + form[dateFieldName + "_mm"].value + form[dateFieldName + "_dd"].value;
    },
    setFormDate: function(form, dateFieldName, dateFieldValue) {
        var result = /(\d{4})(\d{2})(\d{2})/.exec(dateFieldValue);
        if (result) {
            form[dateFieldName + "_yyyy"].value = result[1];
            form[dateFieldName + "_mm"].value = result[2];
            form[dateFieldName + "_dd"].value = result[3];
        }
    },
    log: function() {
        if (typeof console != "undefined" && typeof console.log == "function") {
            console.log.apply(console, arguments);
        }
    },


    trim: function(stringToTrim) {
	return stringToTrim.replace(/^\s+|\s+$/g,"");
    },
    ltrim: function(stringToTrim) {
	return stringToTrim.replace(/^\s+/,"");
    },
    rtrim: function(stringToTrim) {
	return stringToTrim.replace(/\s+$/,"");
    },
};

PFLEX.protocol = (d.location.protocol.indexOf("http") === 0) ? d.location.protocol.split(":")[0] : "http";

PFLEX.stats = "@adserverweb_stats_domain@";

PFLEX.prefix = (PFLEX.protocol == "https") ? "s": "";

PFLEX.locale = "en";

PFLEX.defaultLocale = "en";

PFLEX.path = function () {
    if (arguments.length == 2) {
        return arguments[1] + '/';
    }
    if (arguments.length == 3) {
        return arguments[1] + '/' + arguments[2] + '/';
    }
    return "";
};

var nObj_ = this.navigator;
var uaString_ = nObj_.userAgent;

PFLEX.browser = { };
PFLEX.browser.version = nObj_.appVersion;
PFLEX.browser.isOpera = uaString_.indexOf('Opera') === 0;
PFLEX.browser.isIe = !PFLEX.browser.isOpera && uaString_.indexOf('MSIE') != -1;
PFLEX.browser.isWebkit = !PFLEX.browser.isOpera && uaString_.indexOf('WebKit') != -1;
PFLEX.browser.isCssSupportOpacity = 
    function() {
        var testdiv = document.createElement("div");
        testdiv.style.display = "none";
        testdiv.innerHTML = "<a style='opacity:.55;'>a<\/a>";
        
        var testlink = testdiv.getElementsByTagName("a")[0];
        return (testlink && /^0.55$/.test( testlink.style.opacity ));
}();

// *** Native Extensions ***

PFLEX.K = function(x) { return x; };

PFLEX.$break = { };

PFLEX.Enumerable = {
    pflexEach: function (iterator, context) {
        var index = 0;
        iterator = PFLEX.bind(iterator, context);
        try {
          this._pflexEach(function(value) {
            iterator(value, index++);
          });
        } catch (e) {
          if (e != PFLEX.$break) throw e;
        }
        return this;
    },
    pflexAll: function (iterator, context) {
        iterator = iterator ? PFLEX.bind(iterator, context) : PFLEX.K;
        var result = true;
        this.pflexEach(function(value, index) {
            result = result && !!iterator(value, index);
            if (!result) throw PFLEX.$break;
        });
        return result;
    },
    pflexAny: function(iterator, context) {
        iterator = iterator ? PFLEX.bind(iterator, context) : PFLEX.K;
        var result = false;
        this.pflexEach(function(value, index) {
            if (result = !!iterator(value, index)) throw PFLEX.$break;
        });
        return result;
    },
    pflexSelect: function (iterator, context) {
        iterator = PFLEX.bind(iterator, context);
        var results = [];
        this.pflexEach(function(value, index) {
            if (iterator(value, index))
                results.push(value);
        });
        return results;
    }
};

PFLEX.extend(Array.prototype, PFLEX.Enumerable);

PFLEX.extend(Array.prototype, {
  _pflexEach: function(iterator) {
    for (var i = 0, length = this.length; i < length; i++)
      iterator(this[i]);
  }
});

PFLEX.ClassNames = PFLEX.create();

PFLEX.ClassNames.prototype = {
    init: function(element) {
        this.element = PFLEX.$(element);
    },
    _pflexEach: function(iterator) {
        this.element.className.split(/\s+/).pflexSelect(function(name) {
            return name.length > 0;
        })._pflexEach(iterator);
    }
};

PFLEX.extend(PFLEX.ClassNames.prototype, PFLEX.Enumerable);

PFLEX.Cookies = PFLEX.create();

PFLEX.extend(PFLEX.Cookies, {
    exists: function(name) {
        return !PFLEX.isStringBlank(PFLEX.Cookies.get(name));
    },
    get: function(name) {
        var r = null, cookies = PFLEX.Cookies.parse();
        if (typeof name === "string") {
            cookie = (typeof cookies[name] !== "undefined" ) ? cookies[name] : null;
        }
        return cookie;
    },
    set: function(name, value, options) {
        if (typeof options !== "object" || options === null) {
            options = {};
        }
        if (typeof value === "undefined" || value === null || typeof value !== "string") {
            value = "";
            options.expiry = -1;
        }
        var cookieOptions = PFLEX.Cookies.makeOptions(options);
        document.cookie = name + "=" + encodeURIComponent(value) + cookieOptions;
    },
    parse: function() {
        var cookies = {}, i, keyval, key, value,
            cookieArray = document.cookie.split(';');
        for (i = 0; i < cookieArray.length; i++) {
            keyval = cookieArray[i].split('=');
            key = keyval[0].replace(/^\s*/,"").replace(/\s*$/,""); //remove leading/trailing whitespace
            try {
                value = decodeURIComponent(keyval[1]);
            } catch(e) {
                value = keyval[1];
            }
            cookies[key] = value;
        }
        return cookies;
    },
    makeOptions: function(options) {
        if (typeof options !== "object" || options === null ) {
             return "; path=" + path;
        }
        var optionsString = "", expiryDate = new Date(),
            expiry = options.expiry,
            path = options.path,
            domain = options.domain,
            secure = options.secure;
        //add expiration date
        if (typeof expiry === "number" && expiry !== 0 ) { // change minutes to date
            expiryDate.setTime(expiryDate.getTime() + (expiry * 60 * 1000));
        }
        if (typeof expiry === "object" && expiry instanceof Date) {
            optionsString = "; expires=" + expiry.toGMTString();
        }
        //set path
        optionsString += (typeof path === "string" && path !== "") ? "; path=" + path : "; path=/";
        //set domain
        optionsString += (typeof domain === "string" && domain !== "") ? "; domain=" + domain : "";
        //set secure
        optionsString += (secure === true ) ? "; secure" : "";
        return optionsString;
    }
});

PFLEX.Coreg = PFLEX.create();

PFLEX.extend(PFLEX.Coreg, {
    cookieNamePrefix: "PFLEX_AD_CR_",
    elements: ["oid", "optin", "cid", "did", "order"],
    cookieOptions: { path: '/', expiry: 60, domain: "@adserverweb_cookie_domain@" },

    exists: function(pid) {
        return PFLEX.Cookies.exists(PFLEX.Coreg.cookieNamePrefix + pid) ? true : false;
    },
    get: function(pid) {
        var cookie,
            offer,
            i = 0,
            j,
            offerData = {};

        cookie = PFLEX.Cookies.get(PFLEX.Coreg.cookieNamePrefix + pid) || "";

        if (cookie != "") {
            cookie = cookie.split(",");
            for (i; i < cookie.length; i++) {
                offer = cookie[i].split("|");
                offerData[offer[0]]={};
                for (j = 0; j < PFLEX.Coreg.elements.length; j++) {
                    offerData[offer[0]][PFLEX.Coreg.elements[j]] = offer[j];
                }
            }
        }
        return offerData;
    },
    set: function(pid, cookie, expiry) {
        var offer,
            cookieArray = [],
            offerArray, i;
                        
        for (offer in cookie) {
            if (cookie.hasOwnProperty(offer)) {
                offerArray = [offer];
                for (i = 1; i < PFLEX.Coreg.elements.length; i++) {
                    offerArray.push(cookie[offer][PFLEX.Coreg.elements[i]]);
                }
                cookieArray.push(offerArray.join("|"));
            }
        }

        PFLEX.Cookies.set(PFLEX.Coreg.cookieNamePrefix + pid, cookieArray.join(","), PFLEX.Coreg.cookieOptions);
    },
    initOffer: function(pid, offerData) {
        var cookie, 
            offer, 
            copy = {}, 
            optin = 0;

        cookie = PFLEX.Coreg.get(pid);
        
        //remove dupes
        for (offer in cookie) {
            if (cookie.hasOwnProperty(offer)) {
                if (offer != offerData.oid) {
                    copy[offer] = cookie[offer];
                } else {
                    optin = cookie[offer].optin;
                }
            }
        }
    
        //add offer
        copy[offerData.oid] = {
            "optin": optin,
            "cid": offerData.cid,
            "did": offerData.did,
            "order": offerData.order
        };
        PFLEX.Coreg.set(pid, copy);
        
        return (optin == 1);
    },
    toggle: function(pid, oid, isOptin) {
        var cookie = PFLEX.Coreg.get(pid);
        if (cookie.hasOwnProperty(oid)) {
            cookie[oid].optin = (isOptin) ? 1 : 0;
        }
        PFLEX.Coreg.set(pid, cookie);
    },
    submit: function(pid) {
        var cookie,
            offer,
            i,
            placement,
            url;

        // Submit placement and offers only if coreg cookie exists
        if (PFLEX.Coreg.exists(pid)) {
            placement = PFLEX.Ads.placement(pid);
            cookie = PFLEX.Coreg.get(pid);

            for (offer in cookie) {
                if (cookie.hasOwnProperty(offer)) {
                    if (cookie[offer].optin == 1) {
                        PFLEX.submit(
                            placement.submitURL,
                            cookie[offer].did,
                            placement.formData, {
                                "pflexoid": offer, 
                                "pflexcrid": cookie[offer].cid
                            },
                            placement.semaphore,
                            placement.subSourceId
                        );
                    }

                    url = PFLEX.stats + "/imp/offer";
                    url += ("/" + offer);
                    if (PFLEX.isString(cookie[offer].cid)) 
                        url += ("/" + cookie[offer].cid);
                    if (PFLEX.isString(placement.subSourceId)) 
                        url += ("/" + placement.subSourceId);
                    PFLEX.renderPixel(url, true);
                }
            }

            url = PFLEX.stats + "/imp/placement";
            url += ("/" + pid);
            if (PFLEX.isString(placement.subSourceId)) 
                url += ("/" + placement.subSourceId);
            PFLEX.renderPixel(url, true);

            PFLEX.Coreg.set(pid, "", -1);
        }
    }
});

PFLEX.console = {
	log: function(messageToLog) {
		console.log(''+messageToLog);
	}
}



// *** Event ***

PFLEX.EventHandler = PFLEX.create();

PFLEX.EventHandler.prototype = {
    init: function (id) {
        this.options = (typeof options == "object") ? options: {};
        this.elementRegistry = {};
        this.registry = {};
        this.id = id;
    },
    registerElement: function (events, elm, oid) {
        var items = [], 
            i, 
            found = false, 
            arr, 
            index = 0;
        if (PFLEX.isElement(elm)) {
            if (PFLEX.isString(events)) {
                items.push(events);
            } else {
                if (PFLEX.isArray(events)) {
                    items = events.slice();
                }
            }
            
            var element = {
                elm: elm, // Element to validate
                oid: oid  // Offer id, this is used for the offer fields (custom fileds linked to an offer)
            };

            for (i = 0; i < items.length; i++) {
                if (this.elementRegistry.hasOwnProperty(items[i])) {
                    // Prevent element to be added more then once
                    arr = this.elementRegistry[items[i]];
                    for (index in arr) {
                        if (arr.hasOwnProperty(index) && arr[index].elm.id == element.elm.id) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        this.elementRegistry[items[i]].push(element);
                    }
                } else {
                    this.elementRegistry[items[i]] = [element];
                }
            }
        }
    },
    getElements: function (event, oid) {
        var type = PFLEX.isString(event) ? event : event.type, 
            items = [],
            arr,
            index;
        if (this.elementRegistry.hasOwnProperty(type)) {
            arr = this.elementRegistry[type];
            for (index in arr) {
                if (arr.hasOwnProperty(index) && arr[index].oid === oid) {
                    items.push(arr[index]);
                }
            }
        }
        return items;
    },
    fire: function (event, parameters) {
        //PFLEX.log("Fired event: ", event);
        var ret = true, array, method, action, params, i, 
            type = PFLEX.isString(event) ? event : event.type;
        
        if (this.registry.hasOwnProperty(type)) {
            array = this.registry[type];
            for (i = 0; i < array.length; i++) {
                action = array[i];
                
                method = action.method;
                // Attach additional params to the already registred params
                params = PFLEX.extend(action.params, parameters ? parameters : {});
                args = action.args;
                args.push(params);

                //PFLEX.log("Running method: ", method, "Args: ", args);
                if (typeof method === "string") {
                    if (!PFLEX.ActionHandler.get(method).run(this, args)) { ret = false; }
                } else {
                    if (!method.apply(this, args)) { ret = false; }
                }

                if (typeof params.onComplete === "string") {
                    this.fire(params.onComplete);
                }
            }
        }

        return ret;
    },
    register: function (event, method, args, parameters) {
        var params = {}, a, k, arr;

        PFLEX.extend(params, parameters ? parameters : {});
        
        arr = PFLEX.isArray(args) ? [event].concat(args) : 
            [event, PFLEX.isString(args) ? args : null, PFLEX.Ads.placement(this.id)];

        var action = {
            method: method, // Name of the action or custom method
            args: arr, // Action arguments
            params: params // Additional params
        };

        if (this.registry.hasOwnProperty(event)) {
            if (!this.actionExists(this.registry[event], action)) {
                this.registry[event].push(action);
            }
        } else {
            this.registry[event] = [action];
        }
    },
    actionExists: function(actions, action) {
        return false;
    }
};

// *** Common storageutil.js***

PFLEX.StorageUtil = {
	addRegistrationData: function(key, value){
		PFLEX.NativeRegistrationStorage.addRegistrationData(key, value);
	},
	getRegistrationData: function(key){
		return PFLEX.NativeRegistrationStorage.getRegistrationData(key);
	},
	clearRegistrationData: function(){
		PFLEX.NativeRegistrationStorage.clearRegistrationData();
	},
	hasValidRegistrationData: function(){
		return PFLEX.NativeRegistrationStorage.hasValidRegistrationData();
	},
	saveOffersState : function() {
		PFLEX.StateStorage.saveOffersState();
    },
    getOffersState : function() {
		return PFLEX.StateStorage.getOffersState();
    },
    clearOffersState: function() {
		PFLEX.StateStorage.clearOffersState();
    },
    saveRegFormState: function() {
		PFLEX.StateStorage.saveRegFormState();
    },
    getRegFormState: function() {
        return PFLEX.StateStorage.getRegFormState();
    },
    clearRegFormState: function() {
        PFLEX.StateStorage.clearRegFormState();
    },
    savePrivacyUrlsState: function() {
        PFLEX.StateStorage.savePrivacyUrlsState();
    },
    getPrivacyUrlsState: function() {
        return PFLEX.StateStorage.getPrivacyUrlsState();
    },
    clearPrivacyUrlsState: function() {
        PFLEX.StateStorage.clearPrivacyUrlsState();
    },
    saveState: function() {
	PFLEX.StateStorage.saveState();
    }
};

PFLEX.StateStorage = {

    saveOffersState : function() {
		var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
		var jsonStr =  JSON.stringify(placement.getOffers());
		PFLEX.NativeStorage.setItem('offers', jsonStr);		
		
    },
    getOffersState : function() {
    		var offersJsonStr = PFLEX.NativeStorage.getItem('offers');
		if(typeof offersJsonStr === "undefined" || offersJsonStr === null) {
			return null;
		} 

		var offersList = JSON.parse(''+offersJsonStr);
		if(typeof offersList === "undefined" || offersList === null) {
			return null;
		}		

		return offersList;
		
    },
    clearOffersState: function() {    	
		PFLEX.NativeStorage.removeItem('offers');	
    },
    saveRegFormState: function() {
		var jsonStr = JSON.stringify(PFLEX.SignUp.getFields());
		PFLEX.NativeStorage.setItem('regFields', jsonStr);		
		
    },
    getRegFormState: function() {
    	var fieldsJsonStr = PFLEX.NativeStorage.getItem('regFields');

		if(typeof fieldsJsonStr === "undefined" || fieldsJsonStr === null) {
			return null;
		} 

		var fieldsList = JSON.parse(''+fieldsJsonStr);
		if(typeof fieldsList === "undefined" || fieldsList === null) {
			return null;
		}		

		return fieldsList;
    },
    clearRegFormState: function() {
		PFLEX.NativeStorage.removeItem('regFields');
    },
    savePrivacyUrlsState: function() {
    },
    getPrivacyUrlsState: function() {
    },
    clearPrivacyUrlsState: function() {
    },
    savePendingOffer: function() {
	var pendingOffer = PFLEX.BaseActions.getPendingOffer();
	var pendingOfferJsonStr = JSON.stringify(pendingOffer);
	PFLEX.NativeStorage.setItem('pendingoffer', pendingOfferJsonStr);
   },
   getPendingOffer: function() {
	var pendingOfferJsonStr = PFLEX.NativeStorage.getItem('pendingoffer');		
	if(typeof pendingOfferJsonStr === "undefined" || pendingOfferJsonStr === null) {
		return null;
	}

	var pendingOfferJson = JSON.parse(pendingOfferJsonStr);
	
	if(typeof pendingOfferJson !== "undefined" || pendingOfferJson !== null) {		
		var offersJsonStr = JSON.stringify(pendingOfferJson.offerjson);
		var selected = pendingOfferJson.selected;

		if(typeof offersJsonStr !== "undefined" && offersJsonStr !== null) { 				
			var offer = new PFLEX.Offer(JSON.parse(offersJsonStr), selected);
			return offer;
		}
	}

	return null;

   },
   clearPendingOffer: function() {
	PFLEX.NativeStorage.removeItem('pendingoffer');
   },
   clearState: function() {
	PFLEX.StateStorage.clearOffersState();
	PFLEX.StateStorage.clearRegFormState();
	PFLEX.StateStorage.clearPrivacyUrlsState();
	PFLEX.PageTransitionState.clearSignupPageType();
	PFLEX.PageTransitionState.clearOffersForMissingInfoPage();
	PFLEX.PageTransitionState.clearPrivacyPageConfig();
	PFLEX.PageTransitionState.clearSavedPage();
    },
    saveState: 	function() {
	PFLEX.PageTransitionState.saveSavedPage();
	var savedPage = PFLEX.BaseActions.getCurrentPage();

	if(savedPage !== "undefined") {
		if(savedPage === "multioffer.html") {
			PFLEX.StateStorage.saveOffersState();
		}
		if(savedPage === "registration.html") {
			PFLEX.StateStorage.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
				privacyPageConfig.setSignUpPageType(signupPageType);
			}
		}
		if(savedPage === "privacy.html") {
			PFLEX.StateStorage.savePrivacyUrlsState();
		}



	}

    }
};



PFLEX.PageTransitionState = {
   	setSignupPageType: function(pageType) {
		PFLEX.NativeStorage.setItem('pageType', pageType);			
	},
   	getSignupPageType: function() {
		return PFLEX.NativeStorage.getItem('pageType');
		
	},
   	clearSignupPageType: function() {
		PFLEX.NativeStorage.removeItem('pageType');
	},
   	setOffersForMissingInfoPage: function(offers) {
		var offersListJsonStr = JSON.stringify(offers);
		PFLEX.NativeStorage.setItem('missingInfoPageOffers', offersListJsonStr);		
	},
   	getOffersForMissingInfoPage: function() {
		var offersListJsonStr = null;
		offersListJsonStr = PFLEX.NativeStorage.getItem('missingInfoPageOffers');
				
		if(typeof offersListJsonStr === "undefined" || offersListJsonStr === null) {
			return null;
		} 

		var offersListJson = JSON.parse(''+offersListJsonStr);
		if(typeof offersListJson === "undefined" || offersListJson === null) {
			return null;
		}

		var offers = [];
		for(var key in offersListJson) {
			var offersJsonStr = JSON.stringify(offersListJson[key].offerjson);
			var selected = offersListJson[key].selected;

			if(typeof offersJsonStr !== "undefined" && offersJsonStr !== null) { 				
				var offer = new PFLEX.Offer(JSON.parse(offersJsonStr), selected);
				offers.push(offer);
			}
		}		

		return offers;

	},
   	clearOffersForMissingInfoPage: function() {
		PFLEX.NativeStorage.removeItem('missingInfoPageOffers');		
	},
	savePrivacyPageConfig: function() {		
	    var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		if(typeof privacyPageConfig !== "undefined" && privacyPageConfig !== null) {
			var jsonStr = JSON.stringify(privacyPageConfig);
			PFLEX.NativeStorage.setItem('privacyPageConfig', ''+jsonStr);
		}
	},
	getPrivacyPageConfig: function() {
		var jsonStr = PFLEX.NativeStorage.getItem('privacyPageConfig');
		if(typeof jsonStr !== "undefined" && jsonStr !== null) {
			var jsonObj = JSON.parse(''+jsonStr);
			//set privacy urls
			var urls = [];
			var urlsJson = JSON.parse(JSON.stringify(jsonObj.privacyUrls));
			for(var key in urlsJson) {
				var targetUrl = urlsJson[key].url;
				if(typeof targetUrl !== "undefined" && targetUrl !== null) {
					var privacyurl = new PFLEX.PrivacyUrl(urlsJson[key].url, urlsJson[key].selected);
					urls.push(privacyurl);					
				}
			}

			var privacyPageConfig = new PFLEX.PrivacyPageConfig(jsonObj.urlToRender, jsonObj.signUpCase, jsonObj.signUpPageType, urls, 
jsonObj.privacyPageType, jsonObj.pageTitle);

			return privacyPageConfig;
		}	
		return null;

    },
    clearPrivacyPageConfig: function() {
	PFLEX.NativeStorage.removeItem('privacyPageConfig');			
    },
    getSavedPage: function() {
	var savedPage = PFLEX.NativeStorage.getItem('savedPage');
	return savedPage;

    },
    saveSavedPage: function() {
	var savedPage = PFLEX.BaseActions.getCurrentPage();
	PFLEX.NativeStorage.setItem('savedPage', savedPage);
    },
    clearSavedPage: function() {
	PFLEX.NativeStorage.removeItem('savedPage');
    }

};

// *** Common AppInfo.js***
// *** <script type="text/javascript" src="./json.js"/> ***


PFLEX.AppInfo = {
    jsonObject: {},
    
    initialize: function(jsonstring) {
	//PFLEX.AppInfo.jsonObject[0] = jsonstring;
	//PFLEX.AppInfo.jsonObject[1] = JSON.parse('' + jsonstring);
    this.jsonObject[0] = jsonstring;
	this.jsonObject[1] = JSON.parse('' + jsonstring);
    },

    getAppInfoPflxAdUrl: function() {
	var jsonObj = this.jsonObject[1];
	return jsonObj.pflexadsUrl;
    },
    getDisplayNameForFieldNamed: function (key) {
        var jsonObj = this.getField(key);
        return jsonObj.displayName;
    },
    getField: function (key) {
        var fields = this.getRegistrationFields();
        var jsonStr = fields[key];
        return JSON.parse('' + jsonStr);
    },
    getFieldCount: function () {
        var fieldCount = 0;
	var fields = this.getRegistrationFields();
	fieldCount = fields.length;
	return fieldCount;
    },
    getOffer: function () {
	var jsonObjForOffer = this.getRegistrationOffer();	
	return new PFLEX.Offer(jsonObjForOffer, "false");
    },
    getOfferString: function (key) {
	var jsonObjForOffer = this.getRegistrationOffer();
	if(typeof jsonObjForOffer === "undefined" || jsonObjForOffer === null)
		return;
	return jsonObjForOffer.key;
    },
    getOrderedFieldNames: function () {
	var result = {};
	var fields = this.getRegistrationFields();
	var i=0;
	for( var key in fields ) {
		result[i] = key;
		i++;
	}
	return result;
    },
    getOrderedFields: function () {	
	var fieldsMap = this.getRegistrationFields();
	var fields = [];
	for( var key in fieldsMap ) {
		var field = PFLEX.extend(fieldsMap[key], { fieldName : key } );
		fields.push(fieldsMap[key]);
	}
	return fields.sort(function(a, b){
			var compValue = a.order - b.order;
			if(compValue == 0) {
				if(PFLEX.isString(a.displayName) & PFLEX.isString(b.displayName)) {
					if(a.displayName == b.displayName){
						return 0;
					}
					return (a.displayName < b.displayName) ? -1 : 1;
				}
			}
			return compValue;
		});
    },
    getPid: function () {
	var jsonObj = this.jsonObject[1];	
	return jsonObj.pid;
    },
    getRegistrationFields: function () {
	var jsonObj = this.jsonObject[1];	
	var fieldsStr =  jsonObj.registrationFields;
        
	return JSON.parse(JSON.stringify(fieldsStr));
    },
    getRegistrationOffer: function () {
	var jsonObj = this.jsonObject[1];		
	var fieldsStr =  jsonObj.registrationOffer;
	if(typeof fieldsStr === "undefined" || fieldsStr === null || fieldsStr === "")
		return;

	return JSON.parse(JSON.stringify(fieldsStr));
    },
    getSubSourceId: function () {
	var jsonObj = this.jsonObject[1];	
	return jsonObj.subSourceId;
    },
    getValidationsForFieldNamed: function (key) {	
	var result = {};	
	var json = this.getField(key);
	var arraysJson =  JSON.parse('' + json.validations);

	for (var i = 0; i < arraysJson.length; i++) {
		result[i] = arraysJson[i];
	}
	return result;
    },
    getValue: function (key) {
	var jsonObj = this.jsonObject[1];		
	return jsonObj.key;
    },
    getAdPath: function () {
	return "ads";
    },
    getJsonPath: function () {
	return "json";
    },
    getAppInfoSubSourceIdPath: function() {
	return "subSourceId";
    },
    getAdUrl: function (offerscount) {
	var url = this.getAppInfoPflxAdUrl() + "/" + this.getAdPath() + "/" + this.getJsonPath() +"/"+this.getPid();
	url = url + "?"+this.getAppInfoSubSourceIdPath()+"="+this.getSubSourceId();
	url = url + "&"+"numOffers="+offerscount;
	url = url + "&"+"version="+"2";
        return url;
    },
    
    setDOMObject: function (id) {
        if (PFLEX.$(id) == "undefined") { return; }
        this._DOMObjectId = id;
    },

    hide: function () {
        if (!this._DOMObjectId || PFLEX.isUndefined(PFLEX.$(this._DOMObjectId))) { return; }
        PFLEX.$(this._DOMObjectId).style.display = "none";
        this.visible = false;
    }
};
/* StringHelper.js 
 * 
 * The library is to show the static text on the ads as per locale. 
 */
PFLEX.StringHelper = PFLEX.create();

PFLEX.extend(PFLEX.StringHelper, {
    add: function (locale, messages) {
        var lm = {};
        PFLEX.$A(messages).pflexEach(function(value) {
            lm[locale + "-" + value[0]] = value[1];
        });
        PFLEX.extend(PFLEX.StringHelper.messages, lm);
    },
    get: function(message) {
        var name;
        if (message) {
            name = PFLEX.locale + "-" + message;
            if (PFLEX.StringHelper.messages[name]) {
                return PFLEX.StringHelper.messages[name];
            }
	    return PFLEX.StringHelper.messages[PFLEX.defaultLocale + "-" + message];
        }
        return PFLEX.StringHelper.messages['_DEFAULT_'];
    },
    messages: {
        '_DEFAULT_':'This value is required.'
    }
});

PFLEX.StringHelper.add('en', 
    [['pontiflex', 'Pontiflex'],
    ['hello_world', 'Hello World'],
    ['decline_offer_text', 'No thanks'],
    ['decline_registration_text', 'No Thanks'],
    ['decline_multioffer_text', 'No Thanks'],
    ['confirm_offer_text', 'Confirm'],
    ['confirm_registration_text', 'Continue'],
    ['confirm_multioffer_text', 'Sign Up'],
    ['privacy1_text', 'Your information is secure. Read our '],
    ['privacy2_text', 'Privacy Policy.'],
    ['thank_you_for_registering', 'Thank you. Your sign up is complete.'],
    ['return_to_application', 'Return to Application'],
    ['multi_offer_title', 'Exclusive Deals From Our Partners'],
    ['missing_info_title', 'Complete Your Signup'],
	['missing_info_header', 'Please fill out the form below to complete your signup.'],
	['missing_info_decline', 'No thanks'],
	['missing_info_confirm', 'Complete'],
	['privacy_policy', 'Privacy Policy'],
	['other_policies', 'Other Policies'],
	['select', 'Select'],
	['close', 'Close'],
	['pontiflex_privacy_policy', 'Pontiflex Privacy Policy'],
	['registration_is_required', '  Registration is required.  '],
	['no_offers_selected', 'Please select at least one offer to sign up or press No Thanks.'],
	['fields_prepopulated', 'For your convenience, we have filled out the email field from your phone.'],
	['signup_ads_by','Signup Ads by'],
	['signup_ads_title','Mobile & Online Signup Ads'],
	['no_offers_available','No offers available in your region.']]
);

PFLEX.StringHelper.add('es',
    [['hello_world', 'Hola a todos'],
    ['decline_offer_text', 'No, gracias'],
    ['decline_registration_text', 'Saltar'],
    ['confirm_offer_text', 'Confirmar'],
    ['confirm_registration_text', 'Continuar']]
);
// *** Placement ***

PFLEX.Placement = PFLEX.create();

PFLEX.Placement.prototype = {
	init: function (pid, options) {
		this.id = pid;
		this.width = 0;
		this.height = 0;
		this.maxOffers = Number.MAX_VALUE;
		this.minOffers = 1;
        this.offerRegistry = {};
        this.adType = null;

        this.options = PFLEX.extend({
            data: {},
            trackingURL: null,
            redirectURL: null,
            redirectTarget: null,
            subSourceId: null,
            numOffers: Number.MAX_VALUE,
            locale: PFLEX.locale
        }, options || {});

        this.formData = this.options.data;
        this.trackingURL = this.options.trackingURL;
        this.redirectURL = this.options.redirectURL;
        this.redirectTarget = this.options.redirectTarget;
        this.subSourceId = this.options.subSourceId;
        this.numOffers = this.options.numOffers;
        this.locale = this.options.locale;
        
		this.totalWeight = 0;
		this.DOMAnchorId = null;
		this.offerDOMAnchorId = null;
	
		this.submitURL = "@appweb_base_url@/bpost";
	
		this._renderTimeout = null;
		this._firstOfferSelected = false;

        this.semaphore = {
            counter: 0,
            max: 0,
            init: function(count){ this.max = count; this.counter = 0; },
            locked: function() { return this.counter < this.max; },
            signal: function() { this.counter++; }
        };
    },
    isAdUnitx: function () {
        if (this.adType == "AdUnitx") {
            return true;
        }
        return false;
    },
    isCoreg: function() {
        if (this.adType == "CoReg") {
            return true;
        }
        return false;
    },
    getDOMAnchor: function () {
        if (!this.DOMAnchorId) { return null; }
        return PFLEX.$(this.DOMAnchorId);
    },
    setDOMAnchor: function (id) {
        if (typeof id != "string") { return; }
        this.DOMAnchorId = id;
    },
    getOfferDOMAnchor: function () {
        if (!this.offerDOMAnchorId) { return this.getDOMAnchor(); }
        return PFLEX.$(this.offerDOMAnchorId);
    },
    setOfferDOMAnchor: function (id) {
        if (typeof id != "string") { return; }
        this.offerDOMAnchorId = id;
    },
    isFirstOfferSelected: function () {
        return this._firstOfferSelected;
    },
    setFirstOfferSelected: function (bool) {
        if (bool === true || bool === false) {
            this._firstOfferSelected = bool;
        }
    },
    writeAnchor: function () {
        var anchor;
        if (this.options.anchorId) {
            anchor = d.getElementById(this.options.anchorId);
            if (anchor) {
                anchor.innerHTML = "";
                anchor.id = this.DOMAnchorId;
            }
        }
        if (!anchor) { d.write("<div id='" + this.DOMAnchorId + "' style='position:relative;'></div>"); }
        this.getDOMAnchor().style.width = this.width + "px";
        this.getDOMAnchor().style.height = this.height + "px";
    }, 
    registerOffer: function (oid) {
        if (PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid] = new PFLEX.Offer(this.id, oid, {
                path: this.path
            });
        }
    }, 
    registerOffer: function (oid, offer) {
        if (PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid] = offer;
        }
    },
    hasSelectedOffer: function () {
        for (var oid in this.offerRegistry) {
            // Added to avoid pflex functions added to the native Object prototype
            if (this.offerRegistry.hasOwnProperty(oid)) {
                var o = this.offerRegistry[oid];
                if (o.loaded && !o.submitted && o.isSelected()) {
                    return true;
                }
            }
        }
        return false;
    },
	getOffers : function () {
		var offers = [];
		for (var oid in this.offerRegistry) {
            		// Added to avoid pflex functions added to the native Object prototype
	            	if (this.offerRegistry.hasOwnProperty(oid)) {
        	    	    var o = this.offerRegistry[oid];
				offers.push(o);
            	    	}
        	}
		return offers;
	},
    getSelectedOffers : function() {
	for (var oid in this.offerRegistry) {
            // Added to avoid pflex functions added to the native Object prototype
            if (this.offerRegistry.hasOwnProperty(oid)) {
                var o = this.offerRegistry[oid];
                if (o.loaded && !o.submitted && o.isSelected()) {
                    return true;
                }
            }
        }
        return false;
    },
    toggleOfferState: function(oid) {
	var offer = this.offerRegistry[oid];
	if(offer) {
		var chkbox = document.getElementById("chkbox_"+oid);
		if(offer.isSelected() === 'true') {
			offer.setSelected('false');
			chkbox.checked = false;
		} else {
			offer.setSelected('true');
			chkbox.checked = true;
		}
	}
    },
    offerCheckIn: function (oid, divId) {
        if (!PFLEX.isUndefined(this.offerRegistry[oid])) {
            this.offerRegistry[oid].setDOMObject(divId);
            this.offerRegistry[oid].loaded = true;
        }
    },
	trackImpression: function () {
		var url;
		if (PFLEX.stats) {
			url = PFLEX.stats + (this.isCoreg() ? "/reg/placement" : "/imp/placement");
			url += ("/" + this.id);
			if (PFLEX.isString(this.subSourceId)) url += ("/" + this.subSourceId);
			PFLEX.renderPixel(url, true);
		}
	}
};
// *** Common offer.js***

PFLEX.Offer = PFLEX.create();

PFLEX.Offer.prototype = {
	init: function (offerjson, selected) {
		this.offerjson = offerjson;
		this.selected = selected;
		this._DOMObjectId = null;
	},
	getText1 : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.offerText1;
	},	
	getText2 : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.offerText2;
	},
	getHeaderText : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.headerText;
	},
	getBodyText : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.bodyText;
	},
	getPrivacyUrl : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.privacyPolicyUrl;
	},
	getImagePath : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.logoPath;
	},
	getOfferJson : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return JSON.stringify(this.offerjson);
		return "";	
	},
	getOid : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.oid;

		return "-1";
	},
	getCreativeId : function() {		
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.creativeId;
	},
	getDealId : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.dealId;
	},
	getConfirmHeader : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.confirmHeader;
	},
	getConfirmBody : function() {
		if(typeof this.offerjson !== "undefined" && this.offerjson !== null)
			return this.offerjson.confirmBody;
	},
	isSelected: function () {
        	return this.selected;
    	},
    	setSelected: function (selected) {
        	this.selected = selected;
    	},
};
// *** Container.js ***

PFLEX.Ads = PFLEX.create();

PFLEX.extend(PFLEX.Ads, {
    addEvent: function (id, options) {
        var ne = { };
        ne[id] = new PFLEX.EventHandler(id, options);
        PFLEX.extend(PFLEX.Ads.events, ne);
    },
    addPlacement: function (id, options) {
        var np = { };
        np[id] = new PFLEX.Placement(id, options);
        PFLEX.extend(PFLEX.Ads.placements, np);
    },
    placement: function(id) {
        return PFLEX.Ads.placements[id] ? PFLEX.Ads.placements[id] : { };
    },
    offer: function(id, oid) {
        return PFLEX.Ads.placement(id).offerRegistry[oid];
    },
    placements: { },
});
// *** Common privacyurl.js***

PFLEX.PrivacyUrl = PFLEX.create();

PFLEX.PrivacyUrl.prototype = {
	init: function(url, selected){
		this.url = url;
		this.selected = selected;
	},
	getUrl: function(){
		return this.url;
	},
	isSelected: function(){
		return selected;
	},
	setSelected: function(selected){
		this.selected = selected;
	}
}

PFLEX.PrivacyPageConfig = PFLEX.create();

PFLEX.PrivacyPageConfig.prototype = {
	init: function(urlToRender, signUpCase, signUpPageType, privacyUrls, privacyPageType, pageTitle){
		this.urlToRender = urlToRender;		
		this.signUpCase = signUpCase;
		this.signUpPageType = signUpPageType;
		this.privacyUrls = privacyUrls;
		this.privacyPageType = privacyPageType;
		this.pageTitle = pageTitle;
	},
	isSignUpCase: function(){
		return this.signUpCase; //true or false
	},
	getSignUpPageType: function(){
		return this.signUpPageType;
	},
	getPrivacyUrls: function(){
		return this.privacyUrls;
	},
	getUrlToRender: function() {
		return this.urlToRender;
	},
	setUrlToRender: function(urlToRender) {
		this.urlToRender = urlToRender;
	},
	getPrivacyPageType: function() {
		return this.privacyPageType; //privacy or privacylist
	},
	setPrivacyUrls: function(privacyUrls){
		this.privacyUrls = privacyUrls;
	},
	setSignUpPageType: function(signUpPageType){
		this.signUpPageType = signUpPageType;
	},
	setPrivacyPageType: function(privacyPageType) {
		this.privacyPageType = privacyPageType;
	},
	getPageTitle: function() {
		return this.pageTitle;
	},
	setPageTitle: function(pageTitle) {
		this.pageTitle = pageTitle;
	},


}

PFLEX.PrivacyActions = PFLEX.create();

PFLEX.extend(PFLEX.PrivacyActions, {

	getPontiflexPrivacyPolicyUrl: function() {
		return "http://www.pflexads.com/privacy_policy.html";
	},

	showPrivacyPage : function(signUpCase) {

		var privacyPageUrl = PFLEX.PrivacyActions.getPontiflexPrivacyPolicyUrl();
		var privacyPageConfig = new PFLEX.PrivacyPageConfig(privacyPageUrl, signUpCase, 'registration', '', 'privacy', PFLEX.StringHelper.get('privacy_policy'));
		if(signUpCase) {
			PFLEX.StorageUtil.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
				privacyPageConfig.setSignUpPageType(signupPageType);
			}

		} else {
			//multioffer page, store urrent offers state
			PFLEX.StorageUtil.saveOffersState();

			//set privacy urls
			var urls = [];
			var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
			var offersList = placement.getOffers();
			for(var key in offersList) {
				var o = offersList[key];
				var selected = o.selected;
				if(typeof selected !== "undefined" && selected !== null) {
					var privacyurl = new PFLEX.PrivacyUrl(o.getPrivacyUrl(), false);
					urls.push(privacyurl);		
				}
			}
			privacyPageConfig.setPrivacyUrls(urls);

		}
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		//load privacy page
		location.replace('./privacy.html');

	},
	performPrivacyCloseButton: function() {
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();	
		var signUpCase = privacyPageConfig.isSignUpCase(); 
		PFLEX.PageTransitionState.clearPrivacyPageConfig();
		if(signUpCase) {
			//load signup page
			location.replace('./registration.html');

		} else {
			//load multioffer page
			location.replace('./multioffer.html');
		}
	},

	showPrivacyListPage : function() {
		//fetch privacyPageConfig
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		privacyPageConfig.setPrivacyPageType('privacylist');
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		location.replace('./privacy.html');
	},
	peformPrivacySelectButton : function() {
		//fetch privacyPageConfig
		var privacyPageConfig = PFLEX.PrivacyActions.getPrivacyPageConfig();
		privacyPageConfig.setPrivacyPageType('privacy');
		
		//set url-to-render
		var urlToRender = PFLEX.PrivacyActions.getPontiflexPrivacyPolicyUrl();
		var privacyurls = privacyPageConfig.getPrivacyUrls();
		for(var key in privacyurls) {
			var targeturl = privacyurls[key].url;
			if(typeof targeturl !== "undefined" && targeturl !== null) {
				if(privacyurls[key].selected) {
					urlToRender = targeturl;		
				}
			}			
		}
		privacyPageConfig.setUrlToRender(urlToRender);
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		location.replace('./privacy.html');
	},
	getPrivacyPageConfig: function() {
		return this.config[0];
	},
	setPrivacyPageConfig: function(privacyPageConfig) {
		this.config[0] = privacyPageConfig;
	},
	config: {},
	performUrlSelection: function(url) {
		var privacyPageConfig = this.config[0];
		var privacyurls = privacyPageConfig.getPrivacyUrls();
		for(var key in privacyurls) {
			var targeturl = privacyurls[key].url;
			if(typeof targeturl !== "undefined" && targeturl !== null) {
				privacyurls[key].selected = false;
				if(targeturl === url) {
					privacyurls[key].selected = true;
				}
			}			
		}
	}
	
});
// *** Common versionhelper.js***

PFLEX.VersionHelper = {

getPflxAdVersion: function() {
	return "2";
},
getPflxSdkVersionCode: function() {
	return encodeURI("2.0.0");
},
getPflxSdkVersionName: function() {
	return "AndroidSDK";
},
getPflxAdSdkVersion: function() {
	var result = PFLEX.NativeVersionHelper.getOSType();
	result = result + " " + this.getOsVersion();
	result = result + " Sdk " + this.getPflxSdkVersionName();

	return result;
},
getAppName: function() {
	return PFLEX.NativeVersionHelper.getAppName();
},
getAppVersionCode: function() {
	return PFLEX.NativeVersionHelper.getAppVersionCode();
},
getAppVersionName: function() {
	return PFLEX.NativeVersionHelper.getAppVersionName();
},
getAppUri: function() {
	var result = "app://"+ encodeURI(this.getAppName());
	result = result +"/"+this.getAppVersionCode();
	result = result +"/"+this.getAppVersionName();

	return result;
},
getOsVersion: function() {
	return PFLEX.NativeVersionHelper.getOsVersion();
},
getDNT: function() {
	return "0";
},
getUserAgent: function() {
	return PFLEX.NativeVersionHelper.getUserAgent();
},
getLocale: function() {
	return PFLEX.NativeVersionHelper.getLocale();
},
getPostalCode: function() {
	return PFLEX.NativeVersionHelper.getPostalCode();
},
getHeaders: function() {
	headers = {};
	headers["X-Pflex-Ad-Version"] = this.getPflxAdVersion();
	headers["X-Pflex-Ad-Sdk-Version"] = this.getPflxSdkVersionCode();
	headers["X-Pflex-Referer"] = this.getAppUri();
	headers["X-Pflex-Agent"] = this.getUserAgent();
	headers["Accept-Language"] = this.getLocale();

	var postalCode = this.getPostalCode();
	if(typeof postalCode !== "undefined" && postalCode !== null) {
		headers["X-Pflex-Postalcode"] = postalCode;
	}
	
	return headers;
}

};
// *** Common Android.js***

PFLEX.NativeRegistrationStorage = {
    addRegistrationData : function(key, value) {
	ActivityStorage.addRegistrationData(key, value);
    },
    getRegistrationData : function(key) {
        return ActivityStorage.getRegistrationData(key);
    },
    clearRegistrationData: function() {
        ActivityStorage.clearRegistrationData();
    },
    hasValidRegistrationData: function() {
        return ActivityStorage.hasValidRegistrationData();
    }
};




PFLEX.Native = {
    finishActivity: function() {
        Activity.finishActivity();
    },
    initializeData: function() {
        Activity.initializeData();
    },
    isOnline: function() { 
	return Activity.isOnline();
    },
    getLocale: function() {
	return Activity.getLocale();
    },
    getAppInfoJsonContent: function() {
	var jsonStr = Activity.getAppInfoJsonContent();
	return jsonStr;
    },
    logMessage: function(messageStr) {
	Activity.logMessage(''+messageStr);
    },
    cancelProgressDialog: function() {
	Activity.cancelProgressDialog();
    },
    getBasePath: function() {
	return Activity.getBasePath();
    },
    confirm: function(msg) {
	return confirm(msg);
    },
    alert: function(msg) {
	return alert(msg);
    }	
				
};

PFLEX.NativeStorage = {
    	clearState: function() {
		PFLEX.StateStorage.clearState();
    	},
	supports_html5_storage: function() {
  		try {
    			return 'localStorage' in window && window['localStorage'] !== null;
  		} catch (e) {
    			return false;
  		}
  	},
        initialize: function() {
		if(typeof localStorage === "undefined") {
			PFLEX.console.log("Html5 localStorage is NOT found");
			localStorage = ActivityStorage;
		}
	},
	getItem: function(key) {
		this.initialize();
		return localStorage.getItem(key);
	},
	removeItem: function(key) {
		this.initialize();
		localStorage.removeItem(key);
	},
	setItem: function(key, value) {
		this.initialize();		
		localStorage.setItem(key, value);
	}
				
};


PFLEX.NativeDeviceData = {
    getDeviceData : function(key) {
        return ActivityDevice.getDeviceData(key);
    },	
    getScreenOrientation: function() {
	//0-undefined, 1-portrait, 2-landscape, 3-square
        return ActivityDevice.getScreenOrientation();	
    },
    getScreenWidth: function() {
        return ActivityDevice.getScreenWidth();	
    },
    getScreenHeight: function() {
        return ActivityDevice.getScreenHeight();	
    },
	
};

PFLEX.NativeVersionHelper = {

	getAppName: function() {
		return ActivityVersionHelper.getAppName();
	},
	getAppVersionCode: function() {
		return ActivityVersionHelper.getAppVersionCode();
	},
	getAppVersionName: function() {
		return ActivityVersionHelper.getAppVersionName();
	},
	getOsVersion: function() {
		return ActivityVersionHelper.getOsVersion();
	},
	getOSType: function() {
		return "Android";
	},
	getUserAgent: function() {
		return ActivityVersionHelper.getUserAgent();
	},
	getLocale: function() {
		return ActivityVersionHelper.getLocale();
	},
	getPostalCode: function() {
		return ActivityVersionHelper.getPostalCode();	
	}
};


//Render placement & offers

PFLEX.renderJson = {};

PFLEX.renderJson.util = {};

PFLEX.renderJson.util.getFormData = function(fieldArray,placement){
		var formData = {};
		
		PFLEX.$A(fieldArray).pflexEach(function(field){
			if(PFLEX.isDateField(field.elm)){ 
				formData[field.elm.getAttribute("name")] = PFLEX.getFormDate(placement.nodes.form,field.elm.getAttribute("name")); 
			}else{
				formData[field.elm.name] = field.elm.value;
			}
		});
	
		return formData;
	};
	
PFLEX.renderJson.util.addValidationClasses = function(input, classArray){
	PFLEX.$A(classArray).pflexEach(function(value){
		PFLEX.addClassName(input, PFLEX.Validation.tokenToClass[value.name]);
	});
};
	
PFLEX.renderJson.event = {};

PFLEX.renderJson.event.saveSignUpForm = function() {
};
	
PFLEX.renderJson.event.placementSubmit = function(formFields){
		var validation = new PFLEX.Validation();

		var valresult = validation.validate(formFields);
		if(typeof valresult !== "undefined" && valresult !== null && valresult !== "") {				
			if(PFLEX.Native.alert(valresult)) {
				//do nothing
			}
		} else {
			var formData = PFLEX.renderJson.util.getFormData(formFields,this);
			var fields = PFLEX.AppInfo.getRegistrationFields();
			for( var key in fields ) {
				var signupField = PFLEX.SignUp.getField(key);
				signupField.setValue(formData[signupField.fieldName]);
			}
			return PFLEX.SignupActions.confirm();
		}	
		return false;
	};
	
PFLEX.renderJson.event.offerCheckboxChange = function(oid){
	        var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid()) ;
		placement.toggleOfferState(oid);
	};

PFLEX.renderJson.event.signUpFieldValueChange = function(fieldName, value){
		var signupField = PFLEX.SignUp.getField(fieldName);
		signupField.setValue(value);
	};
	
PFLEX.renderJson.util.renderField = function(field,fieldObj,id){
	var fieldType = fieldObj.type || "date"; // dates don't specify their type so we'll default to date 
	return PFLEX.renderJson.util.renderField[fieldType](field,fieldObj,id);
};

PFLEX.renderJson.util.renderField.text = function(field,fieldObj,id){
	return PFLEX.createEl({tag:"input",attr:{name:field, type:"text", pflexmessage:fieldObj.displayName, value: fieldObj.value, placeholder: fieldObj.displayName, title: fieldObj.displayName}, id:id});
};
	
PFLEX.renderJson.util.renderField.select = function(field,fieldObj,id){
	var option, select = PFLEX.createEl({tag:"select",attr:{name:field, pflexmessage:fieldObj.displayName},id:id});
	
	for(option in fieldObj.options ){
		select.appendChild(PFLEX.createEl({tag:"option",attr:{value:fieldObj.options[option]},html:option}));
	}
	
	return select;
};
	
	
PFLEX.renderJson.util.renderField.date = function(field,fieldObj,id){
	var i, 
		div = PFLEX.createEl({tag:"div","class":"dateField", attr:{pflexmessage:field,pflexFieldType:"date",name:field},id:id}), 
		monthSelect =  PFLEX.createEl({id:field + "_mm", tag:"select", "class":"month",attr:{name:field + "_mm",pflexFieldType:"month"}}),
		daySelect =  PFLEX.createEl({id:field + "_dd", tag:"select", "class":"day",attr:{name:field + "_dd",pflexFieldType:"day"}}),
		yearSelect =  PFLEX.createEl({id:field + "_yyyy", tag:"select", "class":"year",attr:{name:field + "_yyyy",pflexFieldType:"year"}}),
		months = [{display:"- -", value:""},{display:"Jan", value:"01"},{display:"Feb", value:"02"},{display:"Mar", value:"03"},{display:"Apr", value:"04"},{display:"May", value:"05"},{display:"Jun", value:"06"},{display:"Jul", value:"07"},{display:"Aug", value:"08"},{display:"Sep", value:"09"},{display:"Oct", value:"10"},{display:"Nov", value:"11"},{display:"Dec", value:"12"}];
	
	PFLEX.$(months).pflexEach(function(month,index){
		monthSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:month.value},html:month.display}));
	});
	
	daySelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:""},html:"- -"}));
	for(i=1;i<=31;i++){
		daySelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:i},html:i}));
	}
	
	//mk - must cover possible dates of birth AND child due dates
	yearSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:""},html:"- -"}));
	for(i=new Date().getFullYear() + 2;i>=new Date().getFullYear() - 100;i--){
		yearSelect.appendChild(PFLEX.createEl({tag:"option",attr:{value:i},html:i}));
	}
	
	div.appendChild(monthSelect);
	div.appendChild(daySelect);
	div.appendChild(yearSelect);
	
	return div;
};
	
	
PFLEX.renderJson.renderPlacement = function(placement, isSignUpPage, signUpPageType, parentDiv, signUpRequired){

		var formFields = [], field, fieldDiv, label, input,
		
		mainDiv = PFLEX.createEl({tag:"div","class":"placement"}),
		brandDiv = PFLEX.createEl({tag:"div","class":"brand",id:"brand"}),
		brandA = PFLEX.createEl({tag:"a","class":"brandAclass",attr: {href:"#"}, html:PFLEX.StringHelper.get('signup_ads_by') + " <img src='" + PFLEX.Native.getBasePath() + "images/logo_sm8.png'/></a>"}),
		buttonDiv = PFLEX.createEl({tag:"div","class":"buttonContainer", id:"buttonContainer"}),
		form = PFLEX.createEl({tag:"form"}),

		header = PFLEX.createEl({tag:"div","class":"header",html:placement.headerText}),
		
		mainOfferDiv = PFLEX.createEl({tag:"div","class":"mainOffer"}),
		messagesDiv = PFLEX.createEl({tag:"div","class":"messages",id:"messages"}),
		errorDiv = PFLEX.createEl({tag:"div","class":isSignUpPage === true ? "errors_signup" : "errors" ,id:"errors"}),
		fieldset = PFLEX.createEl({tag:"fieldset"}),
		policy1 = PFLEX.createEl({tag:"p","class":"privacyclass",html:PFLEX.StringHelper.get('privacy1_text'),id:"policy1"}),
		policy2 = PFLEX.createEl({tag:"a","class":"privacyclass",attr: {href:"#"}, html:PFLEX.StringHelper.get('privacy2_text'),id:"policy2"}), /* utf-8 arrow -  \u2192 */
		confirmBtn = PFLEX.createEl({tag:"input", "class": "confirm",attr:{type:"button",value:isSignUpPage === true ? PFLEX.StringHelper.get('confirm_registration_text') : PFLEX.StringHelper.get('confirm_multioffer_text')}}), 
		skipBtn = PFLEX.createEl({tag:"input", "class": "skip",attr:{type:"button",value:isSignUpPage === true ? PFLEX.StringHelper.get('decline_registration_text') : PFLEX.StringHelper.get('decline_multioffer_text')}});

		parentDiv.appendChild(mainDiv);
		
		if(signUpPageType !== 'registration') {
			mainDiv.appendChild(header);
		}else {
			/*
			var headerImage = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ placement.headerImage}});
			var body = PFLEX.createEl({tag:"p","class":"body",html:placement.bodyText});
			mainDiv.appendChild(headerImage);
			mainDiv.appendChild(body);
			*/
			var regOfferDiv = PFLEX.createEl({tag:"div","class":"offer"});
    			var regOfferLabel = PFLEX.createEl({tag:"label",html:placement.headerText});
    			var regOfferBody = PFLEX.createEl({tag:"p",html:placement.bodyText});
    			var regOfferImg = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ placement.headerImage}});

			var regOffer = PFLEX.AppInfo.getRegistrationOffer();
    	
			if(typeof regOffer !== "undefined" && regOffer	!== null) {		
    				regOfferDiv.appendChild(regOfferImg);
    				regOfferDiv.appendChild(regOfferLabel);
    				regOfferDiv.appendChild(regOfferBody);			
			
				//var regOffer = PFLEX.renderJson.offerRender(placement.registrationOffer);
				mainDiv.appendChild(regOfferDiv)
			}		
		}

		mainDiv.appendChild(form);
		
		if(signUpPageType !== "registration"){
			brandDiv.appendChild(brandA);
			mainDiv.appendChild(brandDiv);
		}
		
		if(isSignUpPage === true) {

			var fields = PFLEX.AppInfo.getOrderedFields();

			for( var i=0; i < fields.length; i++ ) {
				var key = fields[i].fieldName;
				var signupField = PFLEX.SignUp.getField(key);
				//load the error messages
				var errorMessage = signupField.getErrorMessage();
				if (typeof errorMessage !== "undefined" && errorMessage !== "undefined") {
					if(errorMessage !== "null" && errorMessage !== "") {
	    	    				errorDiv.innerHTML += errorMessage + "<br>";
					}
            			}
				fieldDiv = PFLEX.createEl({tag:"div"});
				label = PFLEX.createEl({tag:"label",html:signupField.displayName,attr:{ "for": "pflex_:" + signupField.fieldName}});
				input = PFLEX.renderJson.util.renderField(signupField.fieldName, signupField,"pflex_:" + signupField.fieldName);
				input.onchange = function(e){ return PFLEX.renderJson.event.signUpFieldValueChange(e.target.name, e.target.value); };
				input.onkeydown = function tabE(e) { return PFLEX.renderJson.event.signUpFieldValueChange(e.target.name, PFLEX.trim(e.target.value)); };
				input.onkeyup = function tabE(e) { return PFLEX.renderJson.event.signUpFieldValueChange(e.target.name, PFLEX.trim(e.target.value)); };

				input.onkeypress = function tabE(e) {
					PFLEX.renderJson.event.signUpFieldValueChange(e.target.name, PFLEX.trim(e.target.value)); 
					var e=(typeof window.event!='undefined')?window.event:e;// IE : Moz

					if(e.keyCode==13){ 
						for(var i=0;i<formFields.length;i++){
							if(e.target==formFields[i].elm) {
								var ele = (i==formFields.length-1) ? formFields[i].elm : formFields[i+1].elm;
								//fix for v1.5 and v1.6 - remove hint
								if(typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
									ele.value='';			
								}
								ele.focus();
							} 
						} 
						return false; 
					} 
				};
				input.onclick = function tabE(e) { 
					var e=(typeof window.event!='undefined')?window.event:e;// IE : Moz

					
						for(var i=0;i<formFields.length;i++){
							if(e.target==formFields[i].elm) {
								var ele = formFields[i].elm;
								//fix for v1.5 and v1.6 - remove hint
								if(typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
									ele.value='';			
								}

								ele.focus();
							} 
						} 
						return false; 
					 
				};

				input.onfocus = function tabE(e) { 
					var e=(typeof window.event!='undefined')?window.event:e;// IE : Moz
					
						for(var i=0;i<formFields.length;i++){
							if(e.target==formFields[i].elm) {
								var ele = formFields[i].elm;
								//fix for v1.5 and v1.6 - remove hint
								if(typeof ele.value === "undefined" || ele.value === null || ele.value === '') {
									ele.value='';			
								}
							} 
						} 
						return false; 
					 
				};

				PFLEX.renderJson.util.addValidationClasses(input,signupField.validations);
		
				fieldDiv.appendChild(label);
				fieldDiv.appendChild(input);
				fieldset.appendChild(fieldDiv);
			
				// mk - placing in obj for validation compatibility
				formFields.push({ elm : input});
				
				

				//check if any value is populated from device
				var valuePopulatedFromDevice = signupField.isPopulatedFromDevice();
				if(typeof valuePopulatedFromDevice !== "undefined" && valuePopulatedFromDevice === "true") {
					messagesDiv.innerHTML = ''+PFLEX.StringHelper.get('fields_prepopulated');
	
				}

			}

			//form.appendChild(errorDiv);
			form.appendChild(fieldset);
			if(signUpPageType === "missinginfo") {
				document.title = PFLEX.StringHelper.get('missing_info_title');
			} else {
				document.title = PFLEX.VersionHelper.getAppName();
			}
		} else {
			//form.appendChild(errorDiv);
			form.appendChild(mainOfferDiv);
			document.title = PFLEX.StringHelper.get('multi_offer_title');
		}

		form.appendChild(messagesDiv);		

		form.appendChild(policy1);
		policy1.appendChild(policy2);
		
		buttonDiv.appendChild(skipBtn);
		buttonDiv.appendChild(confirmBtn);
		form.appendChild(buttonDiv);

		if(isSignUpPage === true) {
	        	form.onsubmit = function(){ return PFLEX.renderJson.event.placementSubmit(formFields);};
			// mk - hack to make button work in IE6
			confirmBtn.onclick = function(){ form.onsubmit(); return false; };		
			skipBtn.onclick = function(){ PFLEX.SignupActions.finishActivity(); };
			policy2.onclick = function(){ PFLEX.PrivacyActions.showPrivacyPage(true); };
			brandA.onclick = function(){ PFLEX.BaseActions.showSignupAdsPage(true); };

			if(typeof signUpRequired !== "undefined" && signUpRequired === "true") {
				skipBtn.disabled="true";
			}
		} else {
			form.onsubmit = function(){ return PFLEX.MultiofferActions.offersConfirmed(); };
			// mk - hack to make button work in IE6
			confirmBtn.onclick = function(){ form.onsubmit(); return false; };		
			skipBtn.onclick = function(){ PFLEX.MultiofferActions.finishActivity(); };
			policy2.onclick = function(){ PFLEX.PrivacyActions.showPrivacyPage(false); };
			brandA.onclick = function(){ PFLEX.BaseActions.showSignupAdsPage(false); };
			
			policy1.style.visibility = 'hidden';
			policy2.style.visibility = 'hidden';
 			buttonDiv.style.visibility = 'hidden';
			brand.style.visibility = 'hidden';
		}

		return { dom : mainDiv, form: form, mainOfferDiv : mainOfferDiv, config : placement, formFields : formFields};		
	};


PFLEX.renderJson.offerRender = function(offer,index){
	var div = PFLEX.createEl({tag: "div","class": "offer offer" + index}),
    label = PFLEX.createEl({tag:"label",html:offer.getHeaderText(),attr:{ "for": offer.getOid()}});
    input = PFLEX.createEl({tag:"input","class":"checkbox",attr:{type:"checkbox"}, id:"chkbox_"+offer.getOid(), prop: {checked: offer.isSelected() === 'true' ? true : false}}),
    checkboxDiv = PFLEX.createEl({tag:"div","class":"checkboxDiv"}),
    body = PFLEX.createEl({tag:"p",html: offer.getBodyText()}),
    img = PFLEX.createEl({tag:"img",attr:{ 'src': PFLEX.AppInfo.getAppInfoPflxAdUrl()+ offer.getImagePath()}}),
    hr = PFLEX.createEl({tag:'img','class':'hr',attr:{'src':PFLEX.Native.getBasePath()+'images/hr.jpg'}});
    div.appendChild(input);
    div.appendChild(checkboxDiv);
    div.appendChild(img);
    div.appendChild(label);
    div.appendChild(body);
    div.appendChild(hr);
    if(offer.isSelected() === 'true') {
		PFLEX.addClassName(div,"selected");	
    }
    
    var thisOffer = offer;
	div.onclick = function(){ 
		PFLEX.renderJson.event.offerCheckboxChange(thisOffer.getOid());
		var chkbox = document.getElementById("chkbox_"+thisOffer.getOid());
		if(chkbox.checked) {
			PFLEX.addClassName(div,"selected");
		} else {
			PFLEX.removeClassName(div,"selected");
		}

		return;
	};
    
    return { dom : div }; 
};

PFLEX.renderJson.event.offersLoaded = function() {
	document.getElementById('buttonContainer').style.visibility = 'visible'; 
	document.getElementById('policy1').style.visibility = 'visible'; 
	document.getElementById('policy2').style.visibility = 'visible'; 
	document.getElementById('brand').style.visibility = 'visible'; 
}; 

PFLEX.renderJson.Offer = function(offer,config,index){

    var newOffer = PFLEX.renderJson.offerRender(offer,index);

	newOffer.trackImpression = function(){
		var url = PFLEX.stats + "/imp/offer/" + offer.oid;
		if(PFLEX.isString(config.creativeId)){ url += "/" + config.creativeId; }
		if(PFLEX.isString(config.subSourceId)){ url += "/" + config.subSourceId; }
		
		PFLEX.renderPixel(url,true);
	};
    
    return newOffer;
};


PFLEX.renderJson.Placement = function(config, signUpPageType, parentDiv, signUpRequired){
	
	// mk - setting defaults
	config = PFLEX.extend({
		headerText : "",
		headerImage : signUpPageType == 'registration' ? PFLEX.AppInfo.getOffer().getImagePath(): "",
		bodyText : signUpPageType == 'registration' ? PFLEX.AppInfo.getOffer().getBodyText(): "",
		privacyPolicyUrl : "http://www.pflexads.com/privacy_policy.html"
		}, config);
		
	var placement = {
		config : config,
		offers : [],
		submitURL : "@appweb_base_url@/bpost",
		setOffers : function(offerArray){
			PFLEX.$A(offerArray).pflexEach(function(offer,index){
				var parentDiv, newOffer;
				newOffer = PFLEX.renderJson.Offer(offer,this,index);
				this.offers.push(newOffer);
				newOffer.trackImpression();
				this.nodes.mainOfferDiv.appendChild(newOffer.dom);
			},this);
			//send event that offers are loaded
			PFLEX.renderJson.event.offersLoaded();
			
		},
		trackImpression : function() {
			var url = PFLEX.stats + "/imp/placement/" + this.config.pid;
			if(PFLEX.isString(this.config.subSourceId)){ url += "/" + this.config.subSourceId; }
			PFLEX.renderPixel(url,true);
		}
	};
	if(signUpPageType !== null) {
		if(signUpPageType == 'registration') {
			config.headerText = PFLEX.AppInfo.getOffer().getHeaderText();
		} else {
			config.headerText = PFLEX.StringHelper.get('missing_info_header');
		}
	}

	placement.nodes = PFLEX.renderJson.renderPlacement(config, signUpPageType != null, signUpPageType, parentDiv, signUpRequired);

	placement.trackImpression();
	
	
	return placement;
};


PFLEX.renderJson.thankssAndFinishRender = function() {

	var mainDiv = document.getElementById('main'),
	contentDiv = PFLEX.createEl({tag:"div","class":"thanksAndFinish"}),
	buttonDiv = PFLEX.createEl({tag:"div"}),
	closeBtn = PFLEX.createEl({tag:"button",attr:{type:"submit"},html:PFLEX.StringHelper.get('return_to_application')});
	closeBtn.onclick = function(){ PFLEX.Native.finishActivity(); };

	PFLEX.addClassName(mainDiv,"thanks");
	
	contentDiv.innerHTML = ''+PFLEX.StringHelper.get('thank_you_for_registering');
	buttonDiv.appendChild(closeBtn);

	while (mainDiv.hasChildNodes()) {
	    mainDiv.removeChild(mainDiv.lastChild);
	}

	mainDiv.appendChild(contentDiv);
	mainDiv.appendChild(buttonDiv);

};



// **** Common Action.js ***

PFLEX.BaseActions = PFLEX.create();

PFLEX.BaseActions.prototype = {

init: function () {	
}
};

PFLEX.extend(PFLEX.BaseActions, {

currentPage: {},
setCurrentPage: function(pageName) {
	this.currentPage[0] = pageName;
},
getCurrentPage: function(pageName) {
	return this.currentPage[0];
},
resolveCurrentPage: function(pageName) {
	this.setCurrentPage(''+pageName);
	var savedPage = PFLEX.PageTransitionState.getSavedPage();

	PFLEX.PageTransitionState.clearSavedPage();
    if(typeof savedPage !== "undefined" && savedPage !== null && (''+savedPage !== ''+pageName)) { 		
		this.setCurrentPage(savedPage);
		location.replace('./'+this.getCurrentPage());
	}
},
leadsUrl: function () {
	return "https://app.pontiflex.com/bpost";
},
statsUrl: function () {
	return "http://stats.pflexads.com";
},
pendingOffer: {},
setPendingOffer: function(pendingOffer) {
	PFLEX.BaseActions.pendingOffer[0] = pendingOffer;
},
getPendingOffer: function() {
	return PFLEX.BaseActions.pendingOffer[0];
},
postAcceptedOffer: function (offer) {
	
	if(typeof offer === "undefined" || offer === null)
		return;

	var offerid = offer.getOid();
	if(typeof offerid === "undefined" || offerid === null || offerid === "-1")
		return;


	var isOnlineValue = ''+PFLEX.Native.isOnline();
	if(isOnlineValue === "true") {
		var url = PFLEX.BaseActions.leadsUrl();
		url = url + "?";
		var counter = "0";
                var fieldNames = PFLEX.AppInfo.getOrderedFieldNames();
		for(var key in fieldNames) {
			if(counter !== "0") {
				url = url +"&";
			}			
			counter = counter + 1;
			var fieldNameFromKey = fieldNames[key];
			url = url + encodeURI(fieldNameFromKey) + "="+encodeURI(PFLEX.StorageUtil.getRegistrationData(fieldNameFromKey));			
		}
		url = url + "&"+"pontiflexdealid="+encodeURI(offer.getDealId());
		url = url + "&"+"pflexoid="+encodeURI(offer.getOid());
		url = url + "&"+"pflexcrid="+encodeURI(offer.getCreativeId());
		url = url + "&"+"pflexsubsrcid="+encodeURI(PFLEX.AppInfo.getSubSourceId());

		var callbackFunction = function (jsonStr) {

		};

		var headers = PFLEX.VersionHelper.getHeaders();
		PFLEX.receiveMessage(callbackFunction, url, headers); 
	} else {
		PFLEX.BaseActions.setPendingOffer(offer);
		PFLEX.StateStorage.savePendingOffer();
	}
},
handlePendingOffer: function() {
	var pendingOfferFromState = PFLEX.StateStorage.getPendingOffer();
	if(typeof pendingOfferFromState === "undefined" || pendingOfferFromState === null) {
		return;
	}
	var isOnlineValue = ''+PFLEX.Native.isOnline();
	if(isOnlineValue === "true") {
		PFLEX.BaseActions.postAcceptedOffer(pendingOfferFromState);
		PFLEX.StateStorage.clearPendingOffer();
	}
},
trackAppImpression: function () {
	var url = PFLEX.BaseActions.statsUrl() + "/imp/placement/" + PFLEX.AppInfo.getPid();
	var subsourceId = PFLEX.AppInfo.getSubSourceId();
	var timeInMills = ''+new Date().getTime();
	if(typeof subsourceId !== "undefined" && subsourceId !== null && subsourceId !== "") {
		url = url +"/"+encodeURI(subsourceId);
	}
	url = url + "?"+ timeInMills;

	var callbackFunction = function (jsonStr) {
	};

	var headers = PFLEX.VersionHelper.getHeaders();
	if(PFLEX.Native.isOnline()) {
		PFLEX.receiveMessage(callbackFunction, url, headers); 
	}
},

trackImpressions: function () {
	var placement = PFLEX.Ads.placement(PFLEX.AppInfo.getPid());
	var offersList = placement.getOffers();
	for(var key in offersList) {
		var o = offersList[key];
		var selected = o.selected;
		if(typeof selected !== "undefined" && selected !== null) {
			PFLEX.BaseActions.trackImpression(o);
		}
	}
},

trackImpression: function (offer) {
	if(typeof offer === "undefined" || offer === null)
		return;

	var offerid = offer.getOid();
	if(typeof offerid === "undefined" || offerid === null || offerid === "-1")
		return;

	var url = PFLEX.BaseActions.statsUrl() + "/imp/offer/" + encodeURI(offer.getOid()) +"/" + encodeURI(offer.getCreativeId());
	var subsourceId = PFLEX.AppInfo.getSubSourceId();
	var timeInMills = ''+new Date().getTime();
	if(typeof subsourceId !== "undefined" && subsourceId !== null && subsourceId !== "") {
		url = url +"/"+encodeURI(subsourceId);
	}
	url = url + "?"+ timeInMills;

	var callbackFunction = function (jsonStr) {
	};

	var headers = PFLEX.VersionHelper.getHeaders();
	PFLEX.receiveMessage(callbackFunction, url, headers); 

},
thanksAndFinish: function() {
	if(PFLEX.Native.confirm(PFLEX.StringHelper.get('thank_you_for_registering'))) {
		PFLEX.Native.finishActivity();
	} else {
		//do nothing
	}
},
getSignupAdsUrl: function() {
	return "http://www.pflexads.com";
},

showSignupAdsPage : function(signUpCase) {

	var signupAdsUrl = this.getSignupAdsUrl();

		var privacyPageConfig = new PFLEX.PrivacyPageConfig(signupAdsUrl, signUpCase, 'registration', '', 'privacy', PFLEX.StringHelper.get('signup_ads_title'));
		if(signUpCase) {
			PFLEX.StorageUtil.saveRegFormState();
			var signupPageType = PFLEX.SignUp.getPageType();
			if(typeof signupPageType !== "undefined" && signupPageType === "missinginfo") {
				PFLEX.PageTransitionState.setSignupPageType('missinginfo');
				privacyPageConfig.setSignUpPageType(signupPageType);
			}

		} else {
			//multioffer page, store urrent offers state
			PFLEX.StorageUtil.saveOffersState();
		}
		PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);
		//save privacyPageConfig
		PFLEX.PageTransitionState.savePrivacyPageConfig();

		//load privacy page, this will load url set in PrivacyPageConfig which is signupAdsUrl
		location.replace('./privacy.html');
},
setScreenOrientationAndSize: function() {

	var screenOrientation = PFLEX.NativeDeviceData.getScreenOrientation();
	//console.log('============screenOrientation=============='+screenOrientation);	

	//0-undefined, 1-portrait, 2-landscape, 3-square
	if(screenOrientation === 2) {//landscape
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"landscape");
	} else {
		PFLEX.addClassName(document.getElementsByTagName("body")[0],"portrait");
	}
}



});
// *** Footer ***

// *** START ***
}
// *** END ***

})();
// *** Post Message Header header.js***

(function($ns){

// *** Post Message post-message.js***

/* Using AJAX calls for e.g., to load offers, load images from ad server */

var savedCallback;


$ns.receiveMessage = savedReceiveMessage = function(callback, targetUrl, headers) {
   var request = false;
   try {
     request = new XMLHttpRequest();
   } catch (trymicrosoft) {
     try {
       request = new ActiveXObject("Msxml2.XMLHTTP");
     } catch (othermicrosoft) {
       try {
         request = new ActiveXObject("Microsoft.XMLHTTP");
       } catch (failed) {
         request = false;
       }  
     }
   }


   if (!request) {
     PFLEX.Native.alert("Error initializing XMLHttpRequest!");        
   }

   request.open("GET", targetUrl, true);
   if (callback) {
	// Bind the callback.
            savedCallback = function() {		
		if (request.readyState == 4) {
		       if (request.status == 200) {
				clearTimeout(xmlHttpTimeout); 
				var response = request.responseText;
                		callback(''+response);
			}
		}
            };		
   	request.onreadystatechange = savedCallback;
   }

   PFLEX.console.log('post message targetUrl='+targetUrl);

   if(typeof headers !== "undefined") {
   	for(var key in headers) {
		request.setRequestHeader(''+key, ''+headers[key]);
   	}
   }

   request.send(null);
   // Timeout to abort in 30 seconds
   var xmlHttpTimeout=setTimeout(ajaxTimeout,30000);
   function ajaxTimeout(){
   	request.abort();
   	console.log("Pontiflex SDK request timed out after 30 seconds");
	callback();
   }

};

// *** Post Message Footer ***

})(PFLEX);

// *** Init ***

(function() {

var initPage = function(jsonstr){

	
	var jsonstr = PFLEX.Native.getAppInfoJsonContent();

	PFLEX.AppInfo.initialize(jsonstr);
	
	var privacyPageConfig = PFLEX.PageTransitionState.getPrivacyPageConfig();
	
	var privacyPageUrl = PFLEX.PrivacyActions.getPontiflexPrivacyPolicyUrl();
		//parent.content.location.href = privacyPageUrl;
		//window.open(privacyPageUrl,'content');
	
	var urlToRender = privacyPageConfig.getUrlToRender();
	window.open(urlToRender,'content');

	document.title = privacyPageConfig.getPageTitle();	
		
	if(typeof privacyPageConfig === "undefined" || privacyPageConfig === null) {

		privacyPageConfig = new PFLEX.PrivacyPageConfig(privacyPageUrl, true, 'registration', '', 'privacy', PFLEX.StringHelper.get('privacy_policy'));
	}
	PFLEX.PrivacyActions.setPrivacyPageConfig(privacyPageConfig);

	var policySelectDom = document.getElementById("policySelect");
	
	var privacyUrls = privacyPageConfig.getPrivacyUrls();
	if(privacyUrls.length !== 0){
		PFLEX.addClassName(PFLEX.$('main'),'showPolicySelect');
	}

	//mk - this produces an error but works. i spent a lot of time trying to figure out why this was occuring and came up empty
	policySelectDom.appendChild(PFLEX.createEl({tag:"option",html:PFLEX.StringHelper.get('pontiflex') + " " + PFLEX.StringHelper.get('privacy_policy'),attr:{value:privacyPageUrl}}));

	for ( var i=0; i<=privacyUrls.length-1; ++i ){
		policySelectDom.appendChild(PFLEX.createEl({tag:"option",html:PFLEX.StringHelper.get('privacy_policy') + " #" + (i + 1),attr:{value:privacyUrls[i].url}}));
	}

	//render the fields
	//PFLEX.PrivacyActions.renderPrivacyPage();
	PFLEX.Native.cancelProgressDialog();
};

	
	PFLEX.BaseActions.resolveCurrentPage('privacy.html');
	initPage(PFLEX.Native.getAppInfoJsonContent());


})();

